-- get the addon namespace
local addon, ns = ...

-- get the config values
local cfg = ns.cfg
  
-- holder for some lib functions
local lib = CreateFrame("Frame")

lib.classify =function(u)
	local c = UnitClassification(u)
	if(c == 'elite' or c == 'eliterare') then
		return 'e'
	elseif(c == 'worldboss') then
		return 'b'
	else
		return 'o'
	end
end

lib.GetSetAmount = function(set,updateSetItems)
	if not updateSetItems and cfg.SetItems[set] then
		return cfg.SetItems[set]
	end

	if updateSetItems then
		hedlib.ClearTable( cfg.SetItems )
	end

	local amount = 0
	local setData = cfg.Sets[set]
	if setData then
		for _, itemID in ipairs(setData) do
			if IsEquippedItem(itemID) then
				amount = amount + 1
			end
		end
	end
	cfg.SetItems[set] = amount
	return amount
end

lib.hideRuneFrame=function()
	RuneFrame:Hide()
	RuneFrame.Show = function() end
	RuneFrame.SetPoint = function() end
	RuneFrame.ClearAllPoints = function() end
	RuneFrame.SetAllPoints = function() end
	RuneFrame:UnregisterEvent("RUNE_POWER_UPDATE");
	RuneFrame:UnregisterEvent("RUNE_TYPE_UPDATE");
	RuneFrame:UnregisterEvent("PLAYER_ENTERING_WORLD");
	RuneFrame:SetScript("OnEvent", nil);
	RuneFrame:SetParent(nil)
end

--[[lib.UpdateAllBars = function (bars)
	now = GetTime()
	local ptimeleft=0
	for i=1, cfg.max do
		if(not bars[i].cd) then bars[i].cd=0 end
		if(not bars[i].start) then bars[i].start=now end

		if (bars[i].cd>0 and bars[i].start>0) then
			local timeleft=bars[i].cd - (now - bars[i].start)
			if (timeleft-ptimeleft)>0 then
				bars[i]:SetWidth((timeleft-ptimeleft)*cfg.scale)
				ptimeleft=timeleft
			else
				bars[i]:SetWidth(-1)
			end
		else
			bars[i]:SetWidth(-1)
		end
	end
end]]

lib.OnUpdateBar = function (self)
	cfg.now = GetTime()
	if (cfg.now >= Heddframe.bar.lu + cfg.freq or Heddframe.bar.force) then
		if (Heddframe.bar.cd>0 and Heddframe.bar.start>0) then
			local tl=Heddframe.bar.cd - (cfg.now - Heddframe.bar.start)
			if tl>0 then
				Heddframe.bar:SetWidth(tl*cfg.scale)
			else
				Heddframe.bar:SetScript("OnUpdate", nil)
				Heddframe.bar.cd=0
				Heddframe.bar:SetWidth(-1)
			end
		else
			Heddframe.bar:SetScript("OnUpdate", nil)
			Heddframe.bar.cd=0
			Heddframe.bar:SetWidth(-1)
		end
		Heddframe.bar.force=nil
		Heddframe.bar.lu=cfg.now
	end	
end

lib.UpdateBar = function (spell, start, cd)
	Heddframe.bar.start = start or GetTime()
	Heddframe.bar.cd = cd or 0
	if spell then spell.f:Show() end
	Heddframe.bar.force=true
	Heddframe.bar:SetScript("OnUpdate", lib.OnUpdateBar)
end

lib.UpdateSpellCD = function(spell)
	if spell then
		spell.cd = spell.cd or 0
		spell.start = spell.start or cfg.now
		spell.isUsable = spell.isUsable or 1
		spell.notEnoughMana = spell.notEnoughMana or nil
		local pcd,pstart,pisUsable,pnotEnoughMana = spell.cd,spell.start,spell.isUsable,spell.notEnoughMana
		local s, d, en = GetSpellCooldown(spell.name)
		if (GetSpellInfo(spell.name) and s>0 and d>0 and en) then
			spell.cd=d
			spell.start=s
		else
			spell.cd=0
			spell.start=cfg.now
		end
		spell.isUsable,spell.notEnoughMana = IsUsableSpell(spell.name)
		if spell.cd ~= pcd or spell.start ~= pstart or spell.isUsable~=pisUsable or spell.notEnoughMana~=pnotEnoughMana then return(true) end
	end
	return(nil)
end

lib.CheckCDend = function(cooldown,now)
	if cooldown then
		now=now or GetTime()
		cooldown.cd=cooldown.cd or 0
		cooldown.start=cooldown.start or now
		if now>=cooldown.cd+cooldown.start and cooldown.cd>0 then cooldown.cd=0 cooldown.start=now return(true) end
	end
	return(nil)
end

lib.SetSpellIcon = function(sname)
	local f = cfg.spells[sname].f
	local icon = cfg.spells[sname].icon
	f:SetWidth(cfg.isize)
	f:SetHeight(cfg.isize)
	f.texture = _G["HEDD_"..sname.."_text"] or f:CreateTexture("HEDD_"..sname.."_text","BACKGROUND")
	f.texture:SetAllPoints(f)
	f.texture:SetTexture(icon)
	f:Hide()
end

lib.HideFrame = function(f)
	if f then f:Hide() end
end

lib.AddSpell = function(f,spells,sname,ids,noupdate,buff)
	for index,id in ipairs(ids) do
		local name = GetSpellInfo(id)
		if name and (GetSpellInfo(name) or buff) then
			spells[sname]={}
			spells[sname].f = _G["HEDD_"..sname] or CreateFrame("Frame","HEDD_"..sname,f)
			spells[sname].f:SetAllPoints(f)
			
			spells[sname].name, _, spells[sname].icon, spells[sname].powerCost, spells[sname].isFunnel, spells[sname].powerType, spells[sname].castingTime, spells[sname].minRange, spells[sname].maxRange = GetSpellInfo(id)
			
			lib.SetSpellIcon(sname) --spells[sname].f,spells[sname].icon)
			
			if not noupdate then
				lib.UpdateSpellCD(spells[sname])
			end
			
			spells[sname].noupdate=noupdate or nil
			return true
		end
	end
	return false
end

lib.AddRuneSpell = function(f,spells,sname,ids,runemask)
	for index,id in ipairs(ids) do
		local name = GetSpellInfo(id)
		if name and GetSpellInfo(name) then
			spells[sname]={}
			spells[sname].f = _G["HEDD_"..sname] or CreateFrame("Frame","HEDD_"..sname,f)
			spells[sname].f:SetAllPoints(f)
			
			spells[sname].name, _, spells[sname].icon, spells[sname].powerCost, spells[sname].isFunnel, spells[sname].powerType, spells[sname].castingTime, spells[sname].minRange, spells[sname].maxRange = GetSpellInfo(id)
			lib.SetSpellIcon(sname)
--			lib.UpdateSpellCD(spells[sname])
			spells[sname].noupdate=nil
			spells[sname].red = runemask[1]
			spells[sname].green = runemask[2]
			spells[sname].blue = runemask[3]
			spells[sname].numrunes = spells[sname].red + spells[sname].green + spells[sname].blue
--			spells[sname].next = 0
			lib.UpdateSpellCD(spells[sname])
			return true
		end
	end
	return false
end

lib.UpdateTotem = function(totemtype)
	totemtype = totemtype or 1
	if not cfg.totems then cfg.totems={} end
	if not cfg.totems[totemtype] then
		local haveTotem, totemName, startTime, duration, icon = GetTotemInfo(totemtype)
		cfg.totems[totemtype]={}
		cfg.totems[totemtype].haveTotem = haveTotem
		cfg.totems[totemtype].totemName = totemName
		cfg.totems[totemtype].startTime = startTime
		cfg.totems[totemtype].duration = duration
		return true
	end
	local phaveTotem, ptotemName, pstartTime, pduration=cfg.totems[totemtype].haveTotem,cfg.totems[totemtype].totemName,cfg.totems[totemtype].startTime,cfg.totems[totemtype].duration
	cfg.totems[totemtype].haveTotem,cfg.totems[totemtype].totemName,cfg.totems[totemtype].startTime,cfg.totems[totemtype].duration = GetTotemInfo(totemtype)
	
	if phaveTotem~=cfg.totems[totemtype].haveTotem or ptotemName~=cfg.totems[totemtype].totemName or pstartTime~=cfg.totems[totemtype].startTime or pduration~=cfg.totems[totemtype].duration then
		return true
	end
	return nil
end

lib.GetTotem = function(totemtype)
	if not cfg.totems then lib.UpdateTotem(totemtype) end
	if not cfg.totems[totemtype].haveTotem then return 0 end
	
	local tl = cfg.totems[totemtype].duration - (cfg.now-cfg.totems[totemtype].startTime)
	tl = (tl>0) and tl or 0
	
	return tl
end

lib.UpdateAuras = function(aura)
	if aura.name then
		local pduration,pexpire,pstacks=aura.duration,aura.expire,aura.stacks
		if aura.type=="debuff" then
			local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitDebuff(aura.unit, aura.name)
			if aura.name==name and ((aura.caster=="player" and hedlib.isPlayer(unitCaster)) or aura.caster=="any") then
				aura.duration=duration
				aura.expire=expirationTime
				aura.stacks=count or 1
			else
				aura.duration=0
				aura.expire=0
				aura.stacks=0
			end
		else
			local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitBuff(aura.unit, aura.name)
			if aura.name==name and ((aura.caster=="player" and hedlib.isPlayer(unitCaster)) or aura.caster=="any") then
				aura.duration=duration
				aura.expire=expirationTime
				aura.stacks=count or 1
			else
				aura.duration=0
				aura.expire=0
				aura.stacks=0
			end
		end
		
		if pduration~=aura.duration or pexpire~=aura.expire or pstacks~=aura.stacks then return(true) end
	end
	return(nil)
end

lib.AddAura = function(auras,sname,id,atype,unit,caster)
	auras[sname] ={}
	auras[sname].name = GetSpellInfo(id)
	auras[sname].duration=0
	auras[sname].expire=0
	auras[sname].stacks=0
	auras[sname].type=atype
	auras[sname].unit=unit
	auras[sname].caster=caster or "player"
	lib.UpdateAuras(sname)
end

lib.GetAura = function(auras)
	local tl = 0
	local stacks = 0
	for index,buff in ipairs(auras) do
		if cfg.auras[buff] then
			tl=cfg.auras[buff].expire-cfg.now
			tl = (tl>=0) and tl or 0
			if tl>0 then 
				stacks=cfg.auras[buff].stacks
				break
			end
		end
	end
	return tl,stacks
end

--[[lib.UpdateDebuff = function(debuff)
	if debuff.name then
		local pduration,pexpire,pstacks=debuff.duration,debuff.expire,debuff.stacks
		local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitDebuff("target", debuff.name)
		if debuff.name==name and unitCaster=="player" then
			debuff.duration=duration
			debuff.expire=expirationTime
			debuff.stacks=count or 1
		else
			debuff.duration=0
			debuff.expire=0
			debuff.stacks=0
		end
		if pduration~=debuff.duration or pexpire~=debuff.expire or pstacks~=debuff.stacks then return(true) end
	end
	return(nil)
end]]

lib.PrintCD = function(spell)
	print(spell.name.." : "..(spell.cd - (GetTime() - spell.start)))
end

lib.SimpleCDCheck = function (spell,noresourses)
	noresourses=noresourses or nil
	local overlap=cfg.overlap
	cfg.overlap=0
	if cfg.spells[spell] and (noresourses or (cfg.spells[spell].isUsable==1 and not cfg.spells[spell].notEnoughMana)) then
		local tl=cfg.spells[spell].cd - (cfg.now - cfg.spells[spell].start)
		tl = (tl>=0) and tl or 0
		if tl==0 then
			lib.UpdateBar(cfg.spells[spell],cfg.now,0)
			return true
		else
			if cfg.mintime==tl then
				lib.UpdateBar(cfg.spells[spell],cfg.now,tl)
				return true
			else
				if cfg.cmintime>(tl+overlap) then
					cfg.cmintime=tl
				end
			end
		end
	end
	return nil
end

lib.updateRune = function(rune)
	if not rune then return end
	local update = nil
	local runeType = GetRuneType(rune)
	if runeType then
		if not cfg.runes then
			cfg.runes={}
		end
		if not cfg.runes[rune] then
			cfg.runes[rune]={}
			cfg.runes[rune].runeType=runeType
			cfg.runes[rune].start, cfg.runes[rune].duration, cfg.runes[rune].runeReady = GetRuneCooldown(rune)
			update=true
		else
			local pstart,pduration,pruneReady,pruneType = cfg.runes[rune].start, cfg.runes[rune].duration, cfg.runes[rune].runeReady,runeType
			cfg.runes[rune].runeType=runeType
			cfg.runes[rune].start, cfg.runes[rune].duration, cfg.runes[rune].runeReady = GetRuneCooldown(rune)
			if pstart~=cfg.runes[rune].start or pduration~=cfg.runes[rune].duration or pruneReady~=cfg.runes[rune].runeReady or pruneType~=cfg.runes[rune].runeType then
				update=true
			end
		end
	end
	if lib.myonruneupdate then lib.myonruneupdate(rune) end
	return update
end

lib.updateAllRunes = function()
	local update = nil
	local changed
	for i = 1, 6 do
		changed=lib.updateRune(i)
		if changed then update=true end
	end
	return update
end

lib.getreadyRunes = function(runeType,useDeath)
	local allrunes = runeType and nil or true
	local useDeath = useDeath or nil
	local numrunes = 0
	for i = 1, 6 do
		if cfg.runes[i].start == 0 then
			if allrunes or cfg.runes[i].runeType==runeType or (useDeath and cfg.runes[i].runeType==4) then
				numrunes=numrunes+1
			end
		end
	end
	return numrunes
end

lib.getruneCD = function(runeType,useDeath,exclude)
	useDeath = useDeath or nil
	exclude=exclude or {}
	local start,finish
	local arune = 0
	local atl=999
	local numready=0
	if runeType==1 then
		start,finish=1,2
	elseif runeType==2 then
		start,finish=3,4
	else
		start,finish=5,6
	end
	for i = start, finish do
		if numready>0 then return 0,true,exclude end
		if not exclude[i] and cfg.runes[i].runeType==runeType then
			local tl =  cfg.runes[i].duration - (cfg.now - cfg.runes[i].start)
			tl = (tl>=0) and tl or 0
			if tl==0 then
				numready=numready+1
				exclude[i]=true
			else
				if tl<atl then
					atl=tl
					arune=i
				end
			end
		end
	end
	if numready>0 then return 0,true,exclude end
	
	if useDeath then
		for i = 1, 6 do
			if numready>0 then return 0,true,exclude end
			if not exclude[i] and cfg.runes[i].runeType==4 then
				local tl =  cfg.runes[i].duration - (cfg.now - cfg.runes[i].start)
				tl = (tl>=0) and tl or 0
				if tl==0 then
					numready=numready+1
					exclude[i]=true
				else
					if tl<atl then
						atl=tl
						arune=i
					end
				end
			end
		end	
	end
	if numready>0 then return 0,ready,exclude end

	exclude[arune]=true
	return atl,nil,exclude
end

--[[lib.RuneCDCheck = function (spell,useDeath)
	useDeath = useDeath or nil
	local exclude = {}
	
	if cfg.spells[spell] then
		local tl=cfg.spells[spell].cd - (cfg.now - cfg.spells[spell].start)
		tl = (tl>=0) and tl or 0
		
		local rtl,gtl,btl = 0,0,0
		local rnum,gnum,bnum = 0,0,0
		local ready=nil
		
		if cfg.spells[spell].red==1 then
			rtl,ready,exclude=lib.getruneCD(1,useDeath,exclude)
			if ready then rnum = 1 end
		else
			rnum=1
		end
		
		if cfg.spells[spell].green==1 then
			gtl,ready,exclude=lib.getruneCD(2,useDeath,exclude)
			if ready then gnum = 1 end
		else
			gnum=1
		end
		
		if cfg.spells[spell].blue==1 then
			btl,ready,exclude=lib.getruneCD(3,useDeath,exclude)
			if ready then bnum = 1 end
		else
			bnum=2
		end
			
		if rnum>0 and gnum>0 and bnum>0 then
			tl=tl
		else
			if tl<rtl then tl=rtl end
			if tl<gtl then tl=gtl end
			if tl<btl then tl=btl end
		end
		
		if tl==0 then
			lib.UpdateBar(cfg.spells[spell],cfg.now,0)
			cfg.spells[spell].next=tl
			return true
		else
			if cfg.mintime==tl then
				lib.UpdateBar(cfg.spells[spell],cfg.now,tl)
				cfg.spells[spell].next=tl
				return true
			else
				if cfg.cmintime>tl then
					cfg.cmintime=tl
				end
			end
		end
	end
	cfg.spells[spell].next=tl
	return nil
end]]

lib.RuneCDCheck = function (spell,useDeath,twospells)
	useDeath = useDeath or nil
	twospells = twospells or nil

	local exclude = {}
	if cfg.spells[spell] then
		-- local usedrunes = {}
		local tl=cfg.spells[spell].cd - (cfg.now - cfg.spells[spell].start)
		tl = (tl>=0) and tl or 0
		
		local rtl,gtl,btl = 0,0,0
		local rnum,gnum,bnum = 0,0,0
		local ready=nil
		
		for j = 1,2 do
			if cfg.spells[spell].red==1 then
				rtl,ready,exclude=lib.getruneCD(1,useDeath,exclude)
				if ready then rnum = rnum+1 end
			else
				rnum=2
			end
		
			if cfg.spells[spell].green==1 then
				gtl,ready,exclude=lib.getruneCD(2,useDeath,exclude)
				if ready then gnum = gnum+1 end
			else
				gnum=2
			end
		
			if cfg.spells[spell].blue==1 then
				btl,ready,exclude=lib.getruneCD(3,useDeath,exclude)
				if ready then bnum = bnum+1 end
			else
				bnum=2
			end
			
			if not twospells then break end
		end
		
		if twospells and rnum==2 and gnum==2 and bnum==2 then
			lib.UpdateBar(cfg.spells[spell],cfg.now,0)
--			cfg.spells[spell].next=tl
			return true
		end
		
		if rnum>0 and gnum>0 and bnum>0 then
			tl=tl
		else
			if tl<rtl then tl=rtl end
			if tl<gtl then tl=gtl end
			if tl<btl then tl=btl end
		end
		
		if tl==0 then
			lib.UpdateBar(cfg.spells[spell],cfg.now,0)
--			cfg.spells[spell].next=tl
			return true
		else
			if cfg.mintime==tl then
				lib.UpdateBar(cfg.spells[spell],cfg.now,tl)
--				cfg.spells[spell].next=tl
				return true
			else
				if cfg.cmintime>tl then
					cfg.cmintime=tl
				end
			end
		end
	end
--	cfg.spells[spell].next=tl
	return nil
end

lib.CDandPowerCheck = function (spell,power,less,noresourses)
	noresourses=noresourses or nil
	if cfg.spells[spell] and (noresourses or (cfg.spells[spell].isUsable==1 and not cfg.spells[spell].notEnoughMana)) and ((less and cfg.power<power) or (not less and cfg.power>=power)) then
		local tl=cfg.spells[spell].cd - (cfg.now - cfg.spells[spell].start)
		tl = (tl>=0) and tl or 0
		if tl==0 then
			lib.UpdateBar(cfg.spells[spell],cfg.now,0)
			return true
		else
			if cfg.mintime==tl then
				lib.UpdateBar(cfg.spells[spell],cfg.now,tl)
				return true
			else
				if cfg.cmintime>tl then
					cfg.cmintime=tl
				end
			end
		end
	end
	return nil
end

lib.BuffApply = function (spell,buff,noresourses,stacks)
	noresourses=noresourses or nil
	stacks = stacks or 1
	if cfg.spells[spell] and cfg.auras[buff] and (noresourses or (cfg.spells[spell].isUsable==1 and not cfg.spells[spell].notEnoughMana)) and cfg.auras[buff].stacks<=stacks then
		local tl=cfg.spells[spell].cd - (cfg.now - cfg.spells[spell].start)
		tl = (tl>=0) and tl or 0
		local tl2=cfg.auras[buff].expire-cfg.now
		tl2 = (tl2>=0) and tl2 or 0
		if tl2>2 and cfg.auras[buff].stacks==stacks then return nil end
		--local tl2=cfg.buffs[]
		tl = (tl>tl2) and tl or tl2
		if tl==0 then
			lib.UpdateBar(cfg.spells[spell],cfg.now,0)
			return true
		else
			if cfg.mintime==tl then
				lib.UpdateBar(cfg.spells[spell],cfg.now,tl)
				return true
			else
				if cfg.cmintime>tl then
					cfg.cmintime=tl
				end
			end
		end
	end
	return nil
end

lib.NobuffApply = function (spell,buffs)
	if cfg.spells[spell] and (cfg.spells[spell].isUsable==1 and not cfg.spells[spell].notEnoughMana) then
		local tl=cfg.spells[spell].cd - (cfg.now - cfg.spells[spell].start)
		tl = (tl>=0) and tl or 0
		local tlbuff = 0
		for index,buff in ipairs(buffs) do
			if cfg.auras[buff] then
				local tl2=cfg.auras[buff].expire-cfg.now
				tl2 = (tl2>=0) and tl2 or 0
				tlbuff = tl2>tlbuff and tl2 or tlbuff
			end
		end
	
		tl = (tl>tlbuff) and tl or tlbuff
		
		if tl==0 then
			lib.UpdateBar(cfg.spells[spell],cfg.now,0)
			return true
		else
			if cfg.mintime==tl then
				lib.UpdateBar(cfg.spells[spell],cfg.now,tl)
				return true
			else
				if cfg.cmintime>tl then
					cfg.cmintime=tl
				end
			end
		end
	end
	return nil
end

lib.ApplyonAuras = function (spell,buffs)
	if cfg.spells[spell] and cfg.spells[spell].isUsable==1 and not cfg.spells[spell].notEnoughMana then
		local tl=cfg.spells[spell].cd - (cfg.now - cfg.spells[spell].start)
		tl = (tl>=0) and tl or 0
		local tlbuff = 0
		for index,buff in ipairs(buffs) do
			if cfg.auras[buff] then
				local tl2=cfg.auras[buff].expire-cfg.now
				tl2 = (tl2>=0) and tl2 or 0
				tlbuff = tl2>tlbuff and tl2 or tlbuff
			end
		end
		
		if (tl>tlbuff) then return nil end
		
		if tl==0 then
			lib.UpdateBar(cfg.spells[spell],cfg.now,0)
			return true
		else
			if cfg.mintime==tl then
				lib.UpdateBar(cfg.spells[spell],cfg.now,tl)
				return true
			else
				if cfg.cmintime>tl then
					cfg.cmintime=tl
				end
			end
		end
	end
	return nil
end

lib.ApplyonAura = function (spell,buff,stacks)
	if cfg.spells[spell] and cfg.spells[spell].isUsable==1 and not cfg.spells[spell].notEnoughMana and cfg.auras[buff] then
		local tl=cfg.spells[spell].cd - (cfg.now - cfg.spells[spell].start)
		tl = (tl>=0) and tl or 0
		local tl2=cfg.auras[buff].expire-cfg.now
		tl2 = (tl2>=0) and tl2 or 0
		stacks=stacks or 1
		if tl2==0 or cfg.auras[buff].stacks<stacks then return nil end
		if tl==0 then
			lib.UpdateBar(cfg.spells[spell],cfg.now,0)
			return true
		else
			if cfg.mintime==tl then
				lib.UpdateBar(cfg.spells[spell],cfg.now,tl)
				return true
			else
				if cfg.cmintime>tl then
					cfg.cmintime=tl
				end
			end
		end
	end
	return nil
end

lib.UseTotem = function (totem,totemname)
	if cfg.spells[totemname] and cfg.spells[totemname].isUsable==1 and not cfg.spells[totemname].notEnoughMana and totem then
		local tl=cfg.spells[totemname].cd - (cfg.now - cfg.spells[totemname].start)
		tl = (tl>=0) and tl or 0
		
		local tl2=totem.duration - (cfg.now-totem.startTime)
		tl2 = (tl>=0) and tl2 or 0
		
		if tl>tl2 then
			tl2=tl
		end
			
		if tl2==0 then
			lib.UpdateBar(cfg.spells[totemname],cfg.now,0)
			return true
		else
			if cfg.mintime==tl2 then
				lib.UpdateBar(cfg.spells[totemname],cfg.now,tl2)
				return true
			else
				if cfg.cmintime>tl2 then
					cfg.cmintime=tl2
				end
			end
		end
	end
	return nil
end

lib.overlaping = function (spell,buff,overlap)
	overlap = overlap or cfg.overlap
	local tl=cfg.spells[spell].cd - (cfg.now - cfg.spells[spell].start)
	tl = (tl>=0) and tl or 0
	if buff then
		local tl2=cfg.auras[buff].expire-cfg.now
		tl2 = (tl2>=0) and tl2 or 0
		tl = (tl>tl2) and tl or tl2
	end
	local overlaping = nil
	if tl<(cfg.cmintime+overlap) then overlaping = true end
	return overlaping
end

lib.SetPriority = function()
	cfg.now = GetTime()
	for index,name in pairs(cfg.spells) do
		--if cfg.spells[index].name~="melee" then lib.HideFrame(cfg.spells[index].f) end
		lib.HideFrame(cfg.spells[index].f)
	end
	cfg.mintime=0
	cfg.cmintime = cfg.maxmintime
	local noprio=true
	local count,iters=0,0
	while noprio do
		count=count+1
		for key, value in pairs(cfg.plist) do
			if cfg.case[value] then
--				iters=iters+1
				if cfg.case[value]() then
					noprio=nil
					break
				end
			else
				if lib.SimpleCDCheck(value) then
					noprio=nil
					break
				end
			end
		end
		if count>cfg.numcase then
			lib.UpdateBar(nil)
			break
		end
		if noprio==nil then
			break
		end
		cfg.mintime=cfg.cmintime
	end
--	print (count.." "..iters)
	Heddframe.bar.force=true
end

lib.inrange = function(spell)
	spell = cfg.spells[spell] and cfg.spells[spell].name or spell
	local range=IsSpellInRange(spell,"target")
	if range==0 then return nil end
	return true
end

lib.bdcolor = function(f,color)
	if color then
		f:SetBackdropColor(unpack(color))
		f:SetBackdropBorderColor(unpack(color))
		f:Show()
	else
		f:Hide()
	end
end

cfg.combo = 0
lib.ComboUpdate = function()
	local pcombo = cfg.combo
	cfg.combo = GetComboPoints('player', 'target')
	if pcombo~=cfg.combo then
		return true
	end
	return nil
end

lib.HolyPowerUpdate = function()
	local ppower = cfg.holy
	cfg.holy = UnitPower("player" , SPELL_POWER_HOLY_POWER)
	if ppower~=cfg.holy then
		return true
	end
	return nil
end

ns.lib = lib